(function(global, factory) {
  typeof exports === "object" && typeof module !== "undefined" ? factory(exports) : typeof define === "function" && define.amd ? define(["exports"], factory) : (global = typeof globalThis !== "undefined" ? globalThis : global || self, factory(global.GeometryFns = {}));
})(this, function(exports2) {
  "use strict";
  function crossProduct(line1X, line1Y, line2X, line2Y, x1, y1) {
    return (line2X - line1X) * (y1 - line1Y) - (x1 - line1X) * (line2Y - line1Y);
  }
  function max(a, b) {
    return a > b ? a : b;
  }
  function min(a, b) {
    return a < b ? a : b;
  }
  function areIntersected(qx1, qy1, qx2, qy2, lx1, ly1, lx2, ly2) {
    if (max(qx1, qx2) >= min(lx1, lx2) && max(lx1, lx2) >= min(qx1, qx2) && max(qy1, qy2) >= min(ly1, ly2) && max(ly1, ly2) >= min(qy1, qy2)) {
      return crossProduct(qx1, qy1, qx2, qy2, lx1, ly1) * crossProduct(qx1, qy1, qx2, qy2, lx2, ly2) <= 0 && crossProduct(lx1, ly1, lx2, ly2, qx1, qy1) * crossProduct(lx1, ly1, lx2, ly2, qx2, qy2) <= 0;
    }
    return false;
  }
  function centroidOf(polygon) {
    const { length } = polygon;
    if (length < 4 || length % 2 !== 0) {
      return null;
    }
    if (length === 4) {
      return [(polygon[0] + polygon[2]) / 2, (polygon[1] + polygon[3]) / 2];
    }
    if (length === 6) {
      return [(polygon[0] + polygon[2] + polygon[4]) / 3, (polygon[1] + polygon[3] + polygon[5]) / 3];
    }
    let S = 0;
    let GX = 0;
    let GY = 0;
    const n = length - 2;
    for (let i = 0; i < n; i += 2) {
      const x12 = polygon[i];
      const y12 = polygon[i + 1];
      const x2 = polygon[i + 2];
      const y2 = polygon[i + 3];
      S += (x12 * y2 - x2 * y12) / 2;
      GX += (x12 * y2 - x2 * y12) * (x12 + x2);
      GY += (x12 * y2 - x2 * y12) * (y12 + y2);
    }
    const x1 = polygon[0];
    const y1 = polygon[1];
    const x3 = polygon[n];
    const y3 = polygon[n + 1];
    S += (x3 * y1 - x1 * y3) / 2;
    GX += (x3 * y1 - x1 * y3) * (x3 + x1);
    GY += (x3 * y1 - x1 * y3) * (y3 + y1);
    return [GX / S / 6, GY / S / 6];
  }
  function distanceBetween(x1, y1, x2, y2) {
    const dx = x1 - x2;
    const dy = y1 - y2;
    return Math.sqrt(dx * dx + dy * dy);
  }
  function computeCrossProduct(polygon, start, length) {
    const x1 = polygon[start];
    const y1 = polygon[start + 1];
    const j = (start + 2) % length;
    const x2 = polygon[j];
    const y2 = polygon[j + 1];
    const k = (start + 4) % length;
    const x3 = polygon[k];
    const y3 = polygon[k + 1];
    return crossProduct(x1, y1, x2, y2, x3, y3);
  }
  function isConvexPolygon(polygon) {
    const { length } = polygon;
    if (length < 6 || length % 2 !== 0) {
      return false;
    }
    const direction = computeCrossProduct(polygon, 0, length);
    for (let i = 2; i < length; i += 2) {
      const nextDirection = computeCrossProduct(polygon, i, length);
      if (direction * nextDirection <= 0) {
        return false;
      }
    }
    return true;
  }
  function lineCrossedQuadrangle(line, quadrangle) {
    const lLength = line.length;
    const qLength = quadrangle.length;
    if (lLength !== 4 || qLength !== 8) {
      return null;
    }
    const res = [];
    for (let i = 0, k = 0; i < qLength; i += 2, k++) {
      const x1 = quadrangle[i];
      const y1 = quadrangle[i + 1];
      const j = (i + 2) % 8;
      const x2 = quadrangle[j];
      const y2 = quadrangle[j + 1];
      res[k] = areIntersected(x1, y1, x2, y2, line[0], line[1], line[2], line[3]) ? [x1, y1, x2, y2] : null;
    }
    if (res[0] && res[2] && !res[1] && !res[3]) {
      return [res[0], res[2]];
    }
    if (!res[0] && !res[2] && res[1] && res[3]) {
      return [res[1], res[3]];
    }
    return null;
  }
  exports2.areIntersected = areIntersected;
  exports2.centroidOf = centroidOf;
  exports2.crossProduct = crossProduct;
  exports2.distanceBetween = distanceBetween;
  exports2.isConvexPolygon = isConvexPolygon;
  exports2.lineCrossedQuadrangle = lineCrossedQuadrangle;
  Object.defineProperty(exports2, Symbol.toStringTag, { value: "Module" });
});
